package agent

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"net/http"
	"time"

	"github.com/docker/docker/client"
	"github.com/etoilepolaire/agent/internal/config"
)

// Agent représente l'agent Docker
type Agent struct {
	config     config.Config
	client     *client.Client
	httpClient *http.Client
	agentID    string
	ctx        context.Context
	cancel     context.CancelFunc
}

// New crée une nouvelle instance de l'agent
func New(cfg config.Config) (*Agent, error) {
	if err := cfg.Validate(); err != nil {
		return nil, err
	}

	// Création du client Docker
	dockerClient, err := client.NewClientWithOpts(client.FromEnv, client.WithAPIVersionNegotiation())
	if err != nil {
		return nil, fmt.Errorf("erreur lors de la création du client Docker: %v", err)
	}

	ctx, cancel := context.WithCancel(context.Background())

	return &Agent{
		config:     cfg,
		client:     dockerClient,
		httpClient: &http.Client{Timeout: 10 * time.Second},
		ctx:        ctx,
		cancel:     cancel,
	}, nil
}

// Start démarre l'agent
func (a *Agent) Start() error {
	// Enregistrement de l'agent
	if err := a.register(); err != nil {
		return fmt.Errorf("erreur lors de l'enregistrement: %v", err)
	}

	// Démarrage des routines
	go a.heartbeat()
	go a.watchContainers()
	go a.watchLogs()

	return nil
}

// Stop arrête l'agent
func (a *Agent) Stop() error {
	a.cancel()
	return nil
}

// register enregistre l'agent auprès du serveur
func (a *Agent) register() error {
	registration := struct {
		Name          string `json:"name"`
		Hostname      string `json:"hostname"`
		IPAddress     string `json:"ip_address"`
		DockerVersion string `json:"docker_version"`
	}{
		Name:          a.config.Name,
		Hostname:      a.config.Hostname,
		IPAddress:     a.config.IPAddress,
		DockerVersion: a.config.DockerVersion,
	}

	data, err := json.Marshal(registration)
	if err != nil {
		return err
	}

	resp, err := a.httpClient.Post(
		fmt.Sprintf("%s/api/agents/register", a.config.ServerURL),
		"application/json",
		bytes.NewBuffer(data),
	)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("erreur lors de l'enregistrement: %s", resp.Status)
	}

	var result struct {
		ID string `json:"id"`
	}
	if err := json.NewDecoder(resp.Body).Decode(&result); err != nil {
		return err
	}

	a.agentID = result.ID
	return nil
}

// heartbeat envoie régulièrement un heartbeat au serveur
func (a *Agent) heartbeat() {
	ticker := time.NewTicker(a.config.HeartbeatInterval)
	defer ticker.Stop()

	for {
		select {
		case <-a.ctx.Done():
			return
		case <-ticker.C:
			if err := a.sendHeartbeat(); err != nil {
				fmt.Printf("Erreur lors de l'envoi du heartbeat: %v\n", err)
			}
		}
	}
}

// sendHeartbeat envoie un heartbeat au serveur
func (a *Agent) sendHeartbeat() error {
	resp, err := a.httpClient.Post(
		fmt.Sprintf("%s/api/agents/%s/heartbeat", a.config.ServerURL, a.agentID),
		"application/json",
		nil,
	)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("erreur lors de l'envoi du heartbeat: %s", resp.Status)
	}

	return nil
}

// watchContainers surveille les changements dans les conteneurs
func (a *Agent) watchContainers() {
	// TODO: Implémenter la surveillance des conteneurs
}

// watchLogs surveille les logs des conteneurs
func (a *Agent) watchLogs() {
	// TODO: Implémenter la surveillance des logs
} 