package agent

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"net/http"
	"time"

	"github.com/docker/docker/api/types"
	"github.com/docker/docker/api/types/events"
)

// Container représente un conteneur Docker
type Container struct {
	ID      string            `json:"id"`
	Name    string            `json:"name"`
	Image   string            `json:"image"`
	Status  string            `json:"status"`
	Created time.Time         `json:"created"`
	Ports   map[string][]Port `json:"ports"`
	Labels  map[string]string `json:"labels"`
}

// Port représente un port exposé
type Port struct {
	HostIP   string `json:"host_ip"`
	HostPort string `json:"host_port"`
}

// watchContainers surveille les changements dans les conteneurs
func (a *Agent) watchContainers() {
	events, errs := a.client.Events(a.ctx, types.EventsOptions{})

	for {
		select {
		case <-a.ctx.Done():
			return
		case err := <-errs:
			fmt.Printf("Erreur lors de la surveillance des conteneurs: %v\n", err)
		case event := <-events:
			if event.Type == events.ContainerEventType {
				if err := a.handleContainerEvent(event); err != nil {
					fmt.Printf("Erreur lors du traitement de l'événement: %v\n", err)
				}
			}
		}
	}
}

// handleContainerEvent traite un événement de conteneur
func (a *Agent) handleContainerEvent(event events.Message) error {
	// Mise à jour de la liste des conteneurs
	containers, err := a.listContainers()
	if err != nil {
		return err
	}

	// Envoi de la mise à jour au serveur
	data, err := json.Marshal(containers)
	if err != nil {
		return err
	}

	resp, err := a.httpClient.Post(
		fmt.Sprintf("%s/api/agents/%s/containers/update", a.config.ServerURL, a.agentID),
		"application/json",
		bytes.NewBuffer(data),
	)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("erreur lors de la mise à jour des conteneurs: %s", resp.Status)
	}

	return nil
}

// listContainers liste tous les conteneurs
func (a *Agent) listContainers() ([]Container, error) {
	containers, err := a.client.ContainerList(a.ctx, types.ContainerListOptions{All: true})
	if err != nil {
		return nil, err
	}

	var result []Container
	for _, c := range containers {
		container, err := a.client.ContainerInspect(a.ctx, c.ID)
		if err != nil {
			continue
		}

		ports := make(map[string][]Port)
		for containerPort, bindings := range container.NetworkSettings.Ports {
			var portBindings []Port
			for _, binding := range bindings {
				portBindings = append(portBindings, Port{
					HostIP:   binding.HostIP,
					HostPort: binding.HostPort,
				})
			}
			ports[string(containerPort)] = portBindings
		}

		result = append(result, Container{
			ID:      container.ID,
			Name:    container.Name,
			Image:   container.Config.Image,
			Status:  container.State.Status,
			Created: time.Unix(container.Created, 0),
			Ports:   ports,
			Labels:  container.Config.Labels,
		})
	}

	return result, nil
} 