package agent

import (
	"bufio"
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"time"

	"github.com/docker/docker/api/types"
)

// LogEntry représente une entrée de log
type LogEntry struct {
	ContainerID string    `json:"container_id"`
	Timestamp   time.Time `json:"timestamp"`
	Message     string    `json:"message"`
	Stream      string    `json:"stream"`
}

// watchLogs surveille les logs des conteneurs
func (a *Agent) watchLogs() {
	containers, err := a.client.ContainerList(a.ctx, types.ContainerListOptions{})
	if err != nil {
		fmt.Printf("Erreur lors de la récupération des conteneurs: %v\n", err)
		return
	}

	for _, container := range containers {
		go a.watchContainerLogs(container.ID)
	}
}

// watchContainerLogs surveille les logs d'un conteneur spécifique
func (a *Agent) watchContainerLogs(containerID string) {
	options := types.ContainerLogsOptions{
		ShowStdout: true,
		ShowStderr: true,
		Follow:     true,
		Timestamps: true,
	}

	reader, err := a.client.ContainerLogs(a.ctx, containerID, options)
	if err != nil {
		fmt.Printf("Erreur lors de la récupération des logs: %v\n", err)
		return
	}
	defer reader.Close()

	scanner := bufio.NewScanner(reader)
	for scanner.Scan() {
		select {
		case <-a.ctx.Done():
			return
		default:
			line := scanner.Text()
			if err := a.handleLogLine(containerID, line); err != nil {
				fmt.Printf("Erreur lors du traitement du log: %v\n", err)
			}
		}
	}
}

// handleLogLine traite une ligne de log
func (a *Agent) handleLogLine(containerID, line string) error {
	// Le format des logs Docker est : [timestamp] stream message
	// Exemple : [2024-03-31T22:30:00.000000000Z] stdout Hello World

	var stream string
	var message string
	var timestamp time.Time

	// Parse le timestamp
	if len(line) >= 32 {
		timestampStr := line[1:31]
		var err error
		timestamp, err = time.Parse(time.RFC3339Nano, timestampStr)
		if err != nil {
			return err
		}

		// Parse le stream et le message
		if len(line) >= 35 {
			stream = line[33:35]
			message = line[36:]
		}
	}

	logEntry := LogEntry{
		ContainerID: containerID,
		Timestamp:   timestamp,
		Message:     message,
		Stream:      stream,
	}

	// Envoi du log au serveur
	data, err := json.Marshal(logEntry)
	if err != nil {
		return err
	}

	resp, err := a.httpClient.Post(
		fmt.Sprintf("%s/api/containers/%s/logs", a.config.ServerURL, containerID),
		"application/json",
		bytes.NewBuffer(data),
	)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("erreur lors de l'envoi du log: %s", resp.Status)
	}

	return nil
} 